from ..view import View, Data, Action
from .domain import ListDomainsBase, ViewBase, MainTabs
from .schema import (AddUserSchema, EditUserSchema,
                     AddDomainSchemaWithPreselectedUser)
from pyramid.view import view_config
from gocept.directory2.domain import Domain, User
import gocept.directory2.admin.form


class UserTabs(object):

    @property
    def tabs(self):
        return [
            Action('Info', '', 'domain_user_view', id=self.context.id),
            Action('Domains', '', 'domain_list_for_user', id=self.context.id)
        ]


class ViewUserBase(ViewBase):

    def __call__(self):
        self.title = self.context.name
        self.subtitle = self.context.realname
        self.description = self.context.customer_id
        return super(ViewUserBase, self).__call__()


@view_config(route_name='domain_user_view',
             renderer='templates/view.pt')
class UserView(UserTabs, ViewUserBase):
    """Display one user."""

    schema = AddUserSchema

    @property
    def actions(self):
        yield Action('Edit', 'icon-edit', 'domain_user_edit',
                     id=self.context.id)


@view_config(route_name='domain_user_list',
             renderer='templates/user_list.pt')
class ListUsers(MainTabs, View):
    """List known domains."""
    title = 'Domain users'
    actions = [Action('Add user', 'icon-plus', 'domain_user_add')]

    def domain_data(self, domain):
        return {'name': domain.name,
                'url': self.request.route_url('domain_view', id=domain.id)}

    @property
    def users(self):
        for user in User.query().order_by(User.name):
            yield Data(name=user.name,
                       customer_id=user.customer_id,
                       domains=[self.domain_data(d) for d in
                                Domain.query().filter_by(user=user).filter_by(
                                subdomain_of=None).all()],
                       view_url=self.request.route_url(
                           'domain_user_view', id=user.id))


@view_config(route_name='domain_list_for_user',
             renderer='templates/domain_list.pt')
class ListDomainsForUser(UserTabs, ViewUserBase, ListDomainsBase):
    """List domains for one user."""

    @property
    def actions(self):
        yield Action('Add domain', 'icon-plus', 'domain_add_for_user',
                     id=self.context.id)

    def filter(self, query):
        return query.filter(Domain.user == self.context).filter(
            Domain.subdomain_of == None)


@view_config(route_name='domain_user_add')
class AddUser(gocept.directory2.admin.form.Add):
    """Add a new domain."""

    title = 'Add domain user'
    schema = AddUserSchema()
    factory = User
    next_route = 'domain_user_list'


@view_config(route_name='domain_user_edit')
class EditUser(gocept.directory2.admin.form.Edit):
    """Add a new domain."""

    title = 'Edit domain user'
    schema = EditUserSchema()
    factory = User
    next_route = 'domain_user_view'

    @property
    def next_route_kw(self):
        return {'id': self.context.id}


@view_config(route_name='domain_add_for_user')
class AddDomainForUser(gocept.directory2.admin.form.Add):
    """Add a new domain for user."""

    title = 'Add domain'
    schema = AddDomainSchemaWithPreselectedUser()
    factory = Domain
    next_route = 'domain_list_for_user'

    @property
    def next_route_kw(self):
        return {'id': self.context.id}
