from ..ipaddress import IPAddress
from ..network import Network
from ..nic import NIC
from ..vlan import VLAN
from .schema import Relation
from .view import View, Action
from colander import SchemaNode, String, Integer
from pyramid.httpexceptions import HTTPFound
from pyramid.view import view_config
import colander
import gocept.directory2.admin.form


@view_config(route_name='nic_view',
             renderer='templates/nic/view.pt')
class ViewNIC(View):

    @property
    def actions(self):
        yield Action('Edit', 'icon-edit', 'nic_edit', id=self.context.id)
        yield Action(
            'Allocate address', None, 'nic_allocate', id=self.context.id)

    @property
    def title(self):
        return u'NIC %s' % self.context.mac


class EditSchema(colander.Schema):

    mac = SchemaNode(String(), title='MAC address')


class AddSchema(EditSchema):

    vlan = SchemaNode(
        Relation(VLAN, title='title'), title='VLAN')


@view_config(route_name='nic_add')
class Add(gocept.directory2.admin.form.Add):

    title = 'Add NIC'
    schema = AddSchema()
    factory = NIC

    @property
    def next_route(self):
        return 'machine_view_%s' % self.context.class_

    @property
    def next_route_kw(self):
        return {'id': self.context.id}

    def after_add(self, obj):
        try:
            self.context.add_nic(obj)
        except KeyError, e:
            self._convert_to_validation_error(e, self.schema['vlan'])


@view_config(route_name='nic_edit')
class Edit(gocept.directory2.admin.form.Edit):

    title = 'Edit NIC'
    schema = EditSchema()
    next_route = 'nic_view'


class AllocateSchema(colander.Schema):

    network = SchemaNode(Relation(Network, title='network'))
    index = SchemaNode(
        Integer(), missing=None,
        description='Will be filled automatically if empty.')


@view_config(route_name='nic_allocate')
class Allocate(gocept.directory2.admin.form.Edit):

    title = 'Allocate IP address'
    schema = AllocateSchema()
    next_route = 'nic_view'
    buttons = ('allocate',)

    def allocate_success(self, appstruct):
        self.appstruct = appstruct
        index = appstruct['index']
        network = appstruct['network']
        addr = None
        if index is not None:
            ip = network.netaddr[index]
            addr = IPAddress.create(address=str(ip))
        try:
            addr = network.allocate(self.context, addr)
        except RuntimeError, e:
            self._convert_to_validation_error(e)
        self.request.session.flash(
            'Address allocated: %s' % addr.address, 'success')
        return HTTPFound(location=self.request.route_url(
            self.next_route, id=self.context.id))

    def appstruct(self):
        return {}
