from .widget import RequiredSelectWidget
from deform.widget import TextInputWidget
import colander


class Relation(object):
    """Colander relation type."""

    def __init__(self, cls, title='name'):
        self.cls = cls
        self.title_attribute = title

    def serialize(self, node, appstruct):
        if appstruct is None or appstruct is colander.null:
            return colander.null
        # gocept.directory2.db.Object provides the primary key in ``id``.
        return str(appstruct.id)

    def deserialize(self, node, cstruct):
        if not cstruct:
            return colander.null
        return self.cls.find_by_id(cstruct)

    def cstruct_children(self):
        return []

    def widget_maker(self):
        return RequiredSelectWidget(
            values=[(x.id, getattr(x, self.title_attribute))
                    for x in self.cls.query()])


class ReadOnlyNode(colander.SchemaNode):
    """Schema node for read only fields."""

    def __init__(self, *args, **kw):
        kw['missing'] = ''
        kw['widget'] = TextInputWidget(template='readonly/textinput')
        super(ReadOnlyNode, self).__init__(*args, **kw)


def is_ascii(node, value):
    """Validator which assures that value is ASCII."""
    try:
        value.encode('ascii')
    except UnicodeEncodeError:
        raise colander.Invalid(
            node, '"%s" is not ASCII, which is required here.' % value)
