import gocept.directory2.admin
import gocept.directory2.testing
import gocept.testing.assertion
import pkg_resources
import unittest


def getpath(filename):
    return pkg_resources.resource_filename(__name__, filename)


class AdminApplicationNoAuth(gocept.directory2.admin.AdminApplication):

    def configure_auth(self):
        """Do not configure Auth for tests. This leads
        to everything being public."""
        pass


class AdminApplication(gocept.directory2.admin.AdminApplication):

    who_ini_filename = 'who_testing.ini'

    def configure_auth(self):
        """Copy who.ini to who_testing.ini and replace secretfile with secret.
        """
        with open(getpath('who.ini')) as source,\
             open(getpath(self.who_ini_filename), 'w') as target:
            who_contents = source.read().replace(
                'secretfile = .auth_tkt_secret', 'secret = asdf')
            target.write(who_contents)
        super(AdminApplication, self).configure_auth()


APPLICATION_LAYER_NO_AUTH = gocept.directory2.testing.ApplicationLayer(
    AdminApplicationNoAuth)
WSGI_LAYER_NO_AUTH = gocept.directory2.testing.WSGILayer(
    bases=(APPLICATION_LAYER_NO_AUTH,), name='WSGILayerNoAuth')
BROWSER_LAYER_NO_AUTH = gocept.directory2.testing.BrowserLayer(
    bases=(WSGI_LAYER_NO_AUTH,), name='BrowserLayerNoAuth')


APPLICATION_LAYER = gocept.directory2.testing.ApplicationLayer(
    AdminApplication)
WSGI_LAYER = gocept.directory2.testing.WSGILayer(
    bases=(APPLICATION_LAYER,), name='WSGILayer')
BROWSER_LAYER = gocept.directory2.testing.BrowserLayer(
    bases=(WSGI_LAYER,), name='BrowserLayer')


class BrowserTestCase(unittest.TestCase,
                      gocept.testing.assertion.Ellipsis):

    layer = BROWSER_LAYER_NO_AUTH

    @property
    def browser(self):
        return self.layer['browser']
