from pyramid_who.whov2 import WhoV2AuthenticationPolicy
from pyramid.authorization import ACLAuthorizationPolicy
import gocept.directory2.application
import pkg_resources


def get_groups(identity, request):
    """Compute the groups the currently logged-in machine belongs to."""
    groups = []
    node = identity['gocept.directory.api.machine']
    if node.directory_ring == 0:
        groups.append('group:ring0')
    if node.directory_ring == 1:
        groups.append('group:{}'.format(node.resource_group.name))
    return groups


class APIApplication(gocept.directory2.application.ApplicationFactory):

    def configure_auth(self):
        # Views which do not have a permission are not allowed to access:
        # (deny all by default)
        self.config.set_default_permission('not-allowed')
        self.config.set_authentication_policy(
            WhoV2AuthenticationPolicy(
                pkg_resources.resource_filename(__name__, 'who.ini'),
                'basic-auth', get_groups))
        self.config.set_authorization_policy(ACLAuthorizationPolicy())

    def configure(self):
        super(APIApplication, self).configure()
        self.configure_auth()
        config = self.config

        config.add_route('ring0', '/directory',
                         factory='gocept.directory2.api.ring0.RootFactory')
        config.add_route('ring1', '/directory/{resource_group}',
                         factory='gocept.directory2.api.ring1.RootFactory')

        config.scan(package=__name__,
                    ignore=gocept.directory2.application.scan_ignore_tests)


factory = APIApplication()
