import datetime
import time


def datetime_from_xmlrpc(timestamp):
    return datetime.datetime(
        *(time.strptime(timestamp.value, "%Y%m%dT%H:%M:%S")[0:6]))


def node_to_dict(node):
    result = {}
    result['classes'] = list(sorted(node.puppet_classes))
    result['name'] = node.name
    p = result['parameters'] = {}

    # Generic parameters
    p['id'] = node.global_id
    p['location'] = node.location.abbreviation
    p['machine'] = node.class_
    p['environment'] = node.environment
    p['resource_group'] = node.resource_group.name
    p['timezone'] = node.resource_group.timezone
    p['service_description'] = node.service_description or u''

    if node.directory_password is not None:
        p['directory_password'] = node.directory_password
        p['directory_ring'] = node.directory_ring

    # XXX old API, remove after deploying new API
    for nic in node.nics.values():
        vlan = nic.vlan.vlanclass.abbreviation
        p['mac_%s' % vlan] = nic.mac
        for address in nic.addresses:
            if address.netaddr.version == 4:
                # XXX replace with address instead of index
                p['ip4_%s' % vlan] = address.index

    # New API for communication interface/network information for a node.
    p['interfaces'] = interfaces = {}
    for nic in node.nics.values():
        vlan = nic.vlan.vlanclass.abbreviation
        interfaces[vlan] = {}
        interfaces[vlan]['mac'] = nic.mac
        interfaces[vlan]['networks'] = {}
        for net in nic.vlan.networks.values():
            interfaces[vlan]['networks'][net.network] = [
                addr.address for addr in nic.addresses
                if addr.network is net]

    # Virtual machine parameter
    # XXX use adapter
    if node.class_ == 'virtual':
        p['memory'] = node.memory
        p['disk'] = node.disk
        p['cores'] = node.cores
        if node.kvm_host is not None:
            p['kvm_host'] = node.kvm_host.name
        else:
            p['kvm_host'] = ''

    return result


def patch_pyramid_xmlrpc_to_support_none():
    import pyramid_xmlrpc
    import xmlrpclib

    def xmlrpc_marshal(data):
        if isinstance(data, xmlrpclib.Fault):
            return xmlrpclib.dumps(data)
        else:
            return xmlrpclib.dumps((data,), methodresponse=True, allow_none=1)
    pyramid_xmlrpc.xmlrpc_marshal = xmlrpc_marshal

patch_pyramid_xmlrpc_to_support_none()
