from .machine import Node
from sqlalchemy import Column, Integer, Text, ForeignKey
from sqlalchemy.orm.collections import attribute_mapped_collection
import gocept.directory2.db
import grokcore.component as grok
import netaddr
import sqlalchemy.orm


class NIC(gocept.directory2.db.Object):
    """A network interface card."""

    __tablename__ = 'nics'

    # XXX Should be unique, but the existing database isn't.
    mac = Column(Text)

    vlan_id = Column(Integer, ForeignKey('vlans.id'), nullable=False)
    vlan = sqlalchemy.orm.relation('VLAN')

    node_id = Column(Integer, ForeignKey('nodes.id'), nullable=False)
    node = sqlalchemy.orm.relation(
        'Node', uselist=False,
        backref=sqlalchemy.orm.backref(
            'nics', cascade="all, delete-orphan",
            collection_class=attribute_mapped_collection('name')))

    @property
    def name(self):
        return self.vlan.vlanclass.abbreviation

    def delete(self):
        # XXX SQLAlchemy delete() workaround.
        self.addresses = []
        super(NIC, self).delete()

    def generate_mac(self):
        base = netaddr.EUI(
            '02:00:00:%02x:00:00' % self.vlan.vlanclass.number).value
        self.mac = str(netaddr.EUI(
            base + self.node.global_id)).replace('-', ':')


@grok.subscribe(Node, gocept.directory2.db.Added)
def create_default_nics(node, event):
    """Create default NICs for physical machines."""
    for vlan in node.location.vlans.values():
        if vlan.vlanclass.abbreviation not in node.default_nics:
            continue
        nic = NIC.create(vlan=vlan)
        nic.node = node
        if node.generate_default_mac:
            nic.generate_mac()
        # Generate default addresses for all IPv6 networks
        for net in vlan.networks.values():
            if net.netaddr.version == 6:
                net.allocate(nic)
