import gocept.directory2.tests.test_application as basetest
import gocept.directory2.db_domainshuttle
import unittest
import gocept.directory2.testing
import gocept.directory2.application
import transaction
import zope.component
import zope.sqlalchemy


class MarkTestPostgresAsProductionLayer(gocept.directory2.testing.ApplicationLayer):
    """Modifies the testing PostgreSQL database so that it looks like
    the production database."""

    db_utility = gocept.directory2.db.IDatabase

    def _setup_test_runtime(self):
        super(MarkTestPostgresAsProductionLayer, self)._setup_test_runtime()
        db = zope.component.getUtility(self.db_utility)
        db.session.execute('drop table tmp_functest;')
        zope.sqlalchemy.mark_changed(self.db.session)
        transaction.commit()

    def testSetUp(self):
        pass

    def testTearDown(self):
        pass

POSTGRES_AS_PRODUCTION = MarkTestPostgresAsProductionLayer(
        gocept.directory2.application.ApplicationFactory)


class MarkTestMySqlAsProductionLayer(MarkTestPostgresAsProductionLayer):
    """Modifies the testing MySQL database so that it looks like
    the production database."""

    db_utility = gocept.directory2.db_domainshuttle.IDatabase

MYSQL_AS_PRODUCTION = MarkTestMySqlAsProductionLayer(
        gocept.directory2.application.ApplicationFactory)


class TestPostgresDBTesting(unittest.TestCase):

    layer = POSTGRES_AS_PRODUCTION

    def test_testing_db_obj_cannot_be_created_against_production_db(self):
        db = self.layer['db']
        with self.assertRaises(SystemExit):
            gocept.directory2.db.Database(db.dsn, testing=True)


class TestMySQLDBTesting(unittest.TestCase):

    layer = MYSQL_AS_PRODUCTION

    def test_testing_db_obj_cannot_be_created_against_production_db(self):
        db = self.layer['db_domainshuttle']
        with self.assertRaises(SystemExit):
            gocept.directory2.db_domainshuttle.Database(db.dsn, testing=True)


class TestPostgresDBApplication(unittest.TestCase):

    # Because the application invokes both database connections
    # we need to mark mysql as production to see only
    # the error from postgres.
    layer = MYSQL_AS_PRODUCTION

    def test_application_doesnt_run_against_test_db(self):
        app = gocept.directory2.application.ApplicationFactory(
            **{'sqlalchemy.domainshuttle_url':
                    self.layer['db_domainshuttle'].dsn,
               'sqlalchemy.url': self.layer['db'].dsn})
        with self.assertRaises(SystemExit):
            app.setup_runtime()


class TestMysqlDBApplication(TestPostgresDBApplication):

    layer = POSTGRES_AS_PRODUCTION

