import gocept.directory2.testing
import unittest
import zope.component


class NodeTests(unittest.TestCase):
    """Testing ..machine.Node."""

    layer = gocept.directory2.testing.BASE_APPLICATION_LAYER

    def setUp(self):
        from ..machine import Node
        super(NodeTests, self).setUp()
        self.rg = self.layer['rg0']
        self.rg._refresh()
        self.node = Node.create(name='foo', location=self.layer['loc0'],
                                resource_group=self.rg)

    def test_mail_contacts_sends_mail(self):
        from pyramid_mailer.interfaces import IMailer
        from repoze.sendmail.encoding import cleanup_message
        self.rg.technical_contacts = ['bob@example.com']
        self.node.mail_contacts('de', 'Neustart am 3.10.',
                                'wir werden die VM am 3.10. neu starten.')
        mailer = zope.component.getUtility(IMailer)
        mail = mailer.outbox[-1]
        mail_message = cleanup_message(mail.to_message()).as_string()
        self.assertEqual('gocept support <support@gocept.com>', mail.sender)
        self.assertEqual(['bob@example.com', 'admin@gocept.com'],
                         mail.recipients)
        self.assertRegexpMatches(mail_message, r'\nDate: [^\n]+ \+0000\n')
        self.assertIn('Auto-Submitted: auto-generated\n', mail_message)
        self.assertIn('Precedence: bulk\n', mail_message)
        self.assertIn('Subject: foo: Neustart am 3.10.\n', mail_message)
        self.assertIn('Hallo,\n', mail_message)
        self.assertIn('-- =\n', mail_message)  # encoded quoted-printable

    def test_mail_contacts_raises_error_if_no_contacts(self):
        with self.assertRaisesRegexp(
                ValueError, 'Can not send mail: no contacts'):
            self.node.mail_contacts('de', 'Neustart am 3.10.',
                                    'wir werden die VM am 3.10. neu starten.')
